@@ -26,7 +26,6 @@ import tempfile
 import shutil
 from concurrent.futures import ProcessPoolExecutor
 
-
 APP_NAME = "ova-compose"
 
 VMDK_CONVERT = "vmdk-convert"
@@ -68,8 +67,21 @@ def xml_text_element(tag, value):
     return elem
 
 
-def xml_config(key, val):
-    return ET.Element('{%s}Config' % NS_VMW, { '{%s}required' % NS_OVF: 'false', '{%s}key' % NS_VMW: key, '{%s}value' % NS_VMW: val})
+def xml_config(key, value):
+    if type(value) is bool:
+        value = "true" if value else "false"
+    elif type(value) is not str:
+        value = str(value)
+    return ET.Element('{%s}Config' % NS_VMW, { '{%s}required' % NS_OVF: 'false', '{%s}key' % NS_VMW: key, '{%s}value' % NS_VMW: value})
+
+
+def xml_configs(config):
+    ret = []
+    if config is not None:
+        for k,v in config.items():
+            if v is not None:
+                ret.append(xml_config(k, v))
+    return ret
 
 
 class ValidationError(Exception):
@@ -106,7 +118,7 @@ class VssdSystem(VirtualHardware):
     def xml_item(self, element_name):
         attrs = {}
         item = ET.Element('{%s}System' % NS_OVF, attrs)
-        
+
         item.append(self.xml_element('ElementName', element_name))
         item.append(self.xml_element('InstanceID', 0))
         item.append(self.xml_element('VirtualSystemIdentifier', self.identifier))
@@ -118,18 +130,23 @@ class VssdSystem(VirtualHardware):
 class RasdItem(VirtualHardware):
     last_instance_id = 0
     description = "Virtual Hardware Item"
-    config = {}
+    config = None
     connectable = False
     configuration = None
 
-    def __init__(self, subtype=None):
+
+    def __init__(self, subtype=None, config=None):
         RasdItem.last_instance_id += 1
         self.instance_id = RasdItem.last_instance_id
+        if config is not None:
+            if self.config is None:
+                self.config = {}
+            self.config.update(config)
 
 
     @classmethod
     def from_dict(cls, d):
-        return cls()
+        return cls(config=d.get('config'))
 
 
     def connect(self, ovf):
@@ -163,14 +180,13 @@ class RasdItem(VirtualHardware):
             attrs["{%s}configuration" % NS_OVF] = self.configuration
 
         item = ET.Element('{%s}Item' % NS_OVF, attrs)
-        
+
         item.append(self.xml_element('ResourceType', self.resource_type))
         item.append(self.xml_element('InstanceID', self.instance_id))
         item.append(self.xml_element('Description', self.description))
         item.append(self.xml_element('ElementName', element_name))
 
-        for key, val in sorted(self.config.items()):
-            item.append(xml_config(key, val))
+        item.extend(xml_configs(self.config))
 
         if self.connectable:
             item.append(self.xml_element('AutomaticAllocation', "true" if self.connected else "false"))
@@ -183,8 +199,8 @@ class RasdCpus(RasdItem):
     description = "Virtual CPUs"
 
 
-    def __init__(self, num):
-        super().__init__()
+    def __init__(self, num, config=None):
+        super().__init__(config=config)
         self.num = int(num)
 
 
@@ -193,7 +209,7 @@ class RasdCpus(RasdItem):
         if type(d) is int:
             item = cls(d)
         else:
-            item = cls(d['number'])
+            item = cls(d['number'], config=d.get('config'))
         return item
 
 
@@ -201,7 +217,7 @@ class RasdCpus(RasdItem):
         item = super().xml_item(required, element_name)
         item.append(self.xml_element('AllocationUnits', 'hertz * 10^6'))
         item.append(self.xml_element('VirtualQuantity', self.num))
-        
+
         return item
 
 
@@ -210,8 +226,8 @@ class RasdMemory(RasdItem):
     description = "Virtual Memory"
 
 
-    def __init__(self, size):
-        super().__init__()
+    def __init__(self, size, config=None):
+        super().__init__(config=config)
         self.size = int(size)
 
 
@@ -220,7 +236,7 @@ class RasdMemory(RasdItem):
         if type(d) is int:
             item = cls(d)
         else:
-            item = cls(d['size'])
+            item = cls(d['size'], config=d.get('config'))
         return item
 
 
@@ -228,21 +244,21 @@ class RasdMemory(RasdItem):
         item = super().xml_item(required, element_name)
         item.append(self.xml_element('AllocationUnits', 'byte * 2^20'))
         item.append(self.xml_element('VirtualQuantity', self.size))
-        
+
         return item
 
 
 class RasdController(RasdItem):
 
-    def __init__(self, subtype):
-        super().__init__()
+    def __init__(self, subtype, config=None):
+        super().__init__(config=config)
         self.next_child_address = 0
         self.subtype = subtype
 
 
     @classmethod
     def from_dict(cls, d):
-        item = cls(d.get('subtype', None))
+        item = cls(d.get('subtype', None), config=d.get('config'))
         return item
 
 
@@ -256,8 +272,8 @@ class RasdScsiController(RasdController):
     description = "SCSI Controller"
 
 
-    def __init__(self, subtype):
-        super().__init__(subtype)
+    def __init__(self, subtype, config=None):
+        super().__init__(subtype, config=config)
         self.next_child_address = 0
         # TODO: maintain valid settings in a structure instead of code:
         if self.subtype is None:
@@ -271,7 +287,7 @@ class RasdScsiController(RasdController):
     def xml_item(self, required, element_name):
         item = super().xml_item(required, element_name)
         item.append(self.xml_element('ResourceSubType', self.subtype))
-        
+
         return item
 
 
@@ -295,7 +311,7 @@ class RasdNvmeController(RasdController):
     def xml_item(self, required, element_name):
         item = super().xml_item(required, element_name)
         item.append(self.xml_element('ResourceSubType', 'vmware.nvme.controller'))
-        
+
         return item
 
 
@@ -306,16 +322,15 @@ class RasdIdeController(RasdController):
 
 class RasdControllerItem(RasdItem):
 
-
-    def __init__(self, parent_id):
-        super().__init__()
+    def __init__(self, parent_id, config=None):
+        super().__init__(config=config)
         self.parent_id = parent_id
         self.rasd_parent = None
 
 
     @classmethod
     def from_dict(cls, d):
-        item = cls(d['parent'])
+        item = cls(d['parent'], config=d.get('config'))
         return item
 
 
@@ -341,16 +356,16 @@ class RasdCdDrive(RasdControllerItem):
     connectable = True
 
 
-    def __init__(self, parent_id, image, connected=False):
-        super().__init__(parent_id)
-        self.image = image
+    def __init__(self, parent_id, image, connected=False, config=None):
         self.config = self.DEFAULT_CONFIG.copy()
+        super().__init__(parent_id, config=config)
+        self.image = image
         self.connected = connected
 
 
     @classmethod
     def from_dict(cls, d):
-        item = cls(d['parent'], d.get('image', None), d.get('connected', False))
+        item = cls(d['parent'], d.get('image', None), connected=d.get('connected', False), config=d.get('config'))
         return item
 
 
@@ -406,16 +421,16 @@ class RasdFloppy(RasdItem):
     connectable = True
 
 
-    def __init__(self, image, connected=False):
-        super().__init__()
-        self.image = image
+    def __init__(self, image, connected=False, config=None):
         self.config = self.DEFAULT_CONFIG.copy()
+        super().__init__(config=config)
+        self.image = image
         self.connected = connected
 
 
     @classmethod
     def from_dict(cls, d):
-        item = cls(d.get('image', None), d.get('connected', False))
+        item = cls(d.get('image', None), d.get('connected', False), config=d.get('config'))
         return item
 
 
@@ -440,9 +455,9 @@ class RasdVideoCard(RasdItem):
     connected = False
 
 
-    def __init__(self):
-        super().__init__()
+    def __init__(self, config=None):
         self.config = RasdVideoCard.DEFAULT_CONFIG.copy()
+        super().__init__(config=config)
 
 
     def xml_item(self, required, element_name):
@@ -456,14 +471,23 @@ class RasdHardDisk(RasdControllerItem):
     description = "Hard Disk"
 
 
-    def __init__(self, parent_id, disk):
-        super().__init__(parent_id)
+    def __init__(self, parent_id, disk, config=None):
+        if config is None or 'virtualDiskFormat' not in config:
+            if disk.sector_size is not None:
+                if config is None:
+                    config = {}
+                try:
+                    config['virtualDiskFormat'] = {512: "native_512", 4096: "native_4k"}[disk.sector_size]
+                except KeyError:
+                    print(f"invalid sector size {disk.sector_size}")
+                    raise
+        super().__init__(parent_id, config=config)
         self.disk = disk
 
 
     @classmethod
     def from_dict(cls, d):
-        item = cls(d['parent'], d['disk'])
+        item = cls(d['parent'], d['disk'], config=d.get('config'))
         return item
 
 
@@ -484,9 +508,9 @@ class RasdEthernet(RasdItem):
     connectable = True
 
 
-    def __init__(self, network_id, subtype, connected=True, address=None):
-        super().__init__()
+    def __init__(self, network_id, subtype, connected=True, address=None, config=None):
         self.config = RasdEthernet.DEFAULT_CONFIG.copy()
+        super().__init__(config=config)
         self.network_id = network_id
         self.subtype = subtype
         self.connected = connected
@@ -495,7 +519,7 @@ class RasdEthernet(RasdItem):
 
     @classmethod
     def from_dict(cls, d):
-        item = cls(d['network'], d['subtype'], d.get('connected', True), d.get('address', None))
+        item = cls(d['network'], d['subtype'], connected=d.get('connected', True), address=d.get('address'), config=d.get('config'))
         return item
 
 
@@ -509,7 +533,7 @@ class RasdEthernet(RasdItem):
         item.append(self.xml_element('Connection', self.network.name))
         if self.address:
             item.append(self.xml_element('Address', self.address))
-        
+
         return item
 
 
@@ -530,7 +554,7 @@ class OVFNetwork(object):
         item_desc = ET.Element('{%s}Description' % NS_OVF)
         item_desc.text = f"The {self.name} Network"
         item.append(item_desc)
-        
+
         return item
 
 
@@ -578,7 +602,7 @@ class OVFDisk(object):
             'byte * 2^40' : 2 ** 40,
     }
 
-    def __init__(self, path, units=None, disk_id=None, file_id=None, raw_image=None):
+    def __init__(self, path, units=None, disk_id=None, file_id=None, raw_image=None, sector_size=None):
         if disk_id is None:
             self.id = f"vmdisk{OVFDisk.next_id}"
             OVFDisk.next_id += 1
@@ -598,12 +622,17 @@ class OVFDisk(object):
             assert False, "invalid units used"
         self.units = units
 
+        self.sector_size = int(sector_size) if sector_size is not None else None
+
         if raw_image is not None:
             if os.path.exists(raw_image):
                 # check if the vmdk exists, and if it does if it's newer than the raw image
                 # if not, create vmdk from raw image
                 if not os.path.exists(path) or os.path.getctime(raw_image) > os.path.getctime(path):
-                    subprocess.check_call([VMDK_CONVERT, raw_image, path])
+                    command = [VMDK_CONVERT, raw_image, path]
+                    if sector_size is not None:
+                        command += ["--sector-size", str(sector_size)]
+                    subprocess.check_call(command)
             else:
                 print(f"warning: raw image file {raw_image} does not exist, using {path}")
 
@@ -612,6 +641,15 @@ class OVFDisk(object):
         self.capacity = int(disk_info['capacity'] / self.allocation_factors[self.units])
         self.used = disk_info['used']
 
+        if 'descriptorFile' in disk_info and 'ddb.logicalSectorSize' in disk_info['descriptorFile']:
+            sector_size = int(disk_info['descriptorFile']['ddb.logicalSectorSize'])
+            if self.sector_size is not None:
+                assert self.sector_size == sector_size
+            else:
+                self.sector_size = sector_size
+        elif self.sector_size is None:
+            self.sector_size = 512
+
 
     def host_resource(self):
         return f"ovf:/disk/{self.id}"
@@ -640,7 +678,7 @@ class OVFEmptyDisk(OVFDisk):
         if units in self.allocation_units_map:
             units = self.allocation_units_map[units]
         self.units = units
-
+        self.sector_size = None
 
     def xml_item(self):
         return ET.Element('{%s}Disk' % NS_OVF, {
@@ -661,17 +699,19 @@ class OVFProperty(object):
                  password = False,
                  value=None,
                  user_configurable=False, qualifiers=None,
-                 label=None, description=None, category=None, required=None):
-        self.key = key
-        self.type = type
-        self.password = password
-        self.value = value
-        self.user_configurable = user_configurable
-        self.qualifiers = qualifiers
-        self.label = label
-        self.description = description
+                 label=None, description=None, category=None, required=None, configuration=None):
+
         self.category = category
+        self.configuration = configuration
+        self.description = description
+        self.key = key
+        self.label = label
+        self.password = password
+        self.qualifiers = qualifiers
         self.required = required
+        self.type = type
+        self.user_configurable = user_configurable
+        self.value = value
 
 
     @classmethod
@@ -685,21 +725,47 @@ class OVFProperty(object):
             '{%s}key' % NS_OVF: self.key,
             '{%s}type' % NS_OVF: self.type
         }
+
+        # Handle the default value for the property
+        # DSP0243 2.1.1 line 1026
         if self.value is not None:
-            xml_attrs['{%s}value' % NS_OVF] = str(self.value)
+            # Simple string value (backward compatible)
+            if not isinstance(self.value, dict):
+                xml_attrs['{%s}value' % NS_OVF] = str(self.value)
+            # Dictionary with configuration-specific values
+            elif 'default' in self.value:
+                xml_attrs['{%s}value' % NS_OVF] = str(self.value['default'])
+
+        # Add other standard attributes
         if self.qualifiers is not None:
             xml_attrs['{%s}qualifiers' % NS_OVF] = self.qualifiers
         if self.user_configurable:
             xml_attrs['{%s}userConfigurable' % NS_OVF] = "true"
+        if self.configuration is not None:
+            xml_attrs['{%s}configuration' % NS_OVF] = self.configuration
         if self.password:
             xml_attrs['{%s}password' % NS_OVF] = "true"
         if self.required is not None:
             xml_attrs['{%s}required' % NS_OVF] = "true" if self.required else "false"
+
         xml_property = ET.Element('{%s}Property' % NS_OVF, xml_attrs)
+
+        # Add label and description
         if self.label is not None:
             xml_property.append(xml_text_element('{%s}Label' % NS_OVF, self.label))
         if self.description is not None:
             xml_property.append(xml_text_element('{%s}Description' % NS_OVF, self.description))
+
+        # Add configuration-specific values if present
+        # DSP0243 2.1.1 line 1026
+        if isinstance(self.value, dict) and 'configurations' in self.value:
+            for config_id, config_value in self.value['configurations'].items():
+                config_elem = ET.Element('{%s}Value' % NS_OVF, {
+                    '{%s}value' % NS_OVF: str(config_value),
+                    '{%s}configuration' % NS_OVF: config_id
+                })
+                xml_property.append(config_elem)
+
         return xml_property
 
 
@@ -900,7 +966,8 @@ class OVF(object):
 
 
     def __init__(self,
-                 system, files, disks,
+                 system, hardware_config,
+                 files, disks,
                  networks,
                  vssd_system, rasd_items, extra_configs,
                  products, annotation, eula,
@@ -908,6 +975,10 @@ class OVF(object):
         self.hardware_config = {}
         if not system.get('no_default_configs', False):
             self.hardware_config.update(OVF.CONFIG_DEFAULTS)
+
+        if hardware_config:
+            self.hardware_config.update(hardware_config)
+
         self.name = system['name']
         self.os_cim = system.get('os_cim', 100)
         self.os_vmw = system.get('os_vmw', "other4xLinux64Guest")
@@ -948,6 +1019,7 @@ class OVF(object):
         product = None
         annotation = None
         eula = None
+        hardware_config = {}
 
         assert 'hardware' in config, "config needs a 'hardware' section"
 
@@ -967,7 +1039,8 @@ class OVF(object):
                                    units=hw.get('units', None),
                                    raw_image=hw.get('raw_image', None),
                                    disk_id=hw.get('disk_id', None),
-                                   file_id=hw.get('file_id', None))
+                                   file_id=hw.get('file_id', None),
+                                   sector_size=hw.get('sector_size', None))
                     disks.append(disk)
                     files.append(disk.file)
                     hw['disk'] = disk
@@ -977,6 +1050,8 @@ class OVF(object):
                     disks.append(disk)
                     hw['disk'] = disk
 
+        hardware_config = hardware.get('config', {})
+
         networks = {}
         if 'networks' in config:
             for nw_id, nw in config['networks'].items():
@@ -1021,7 +1096,7 @@ class OVF(object):
             for k, v in config['configurations'].items():
                 configurations[k] = OVFConfiguration(k, **v)
 
-        ovf = cls(config['system'], files, disks,
+        ovf = cls(config['system'], hardware_config, files, disks,
                   networks, vssd_system, rasd_items, extra_configs,
                   products, annotation, eula,
                   configurations)
@@ -1034,7 +1109,9 @@ class OVF(object):
         rasd_items = {}
         hardware = config['hardware']
         for hw_id, hw_config in hardware.items():
-            if hw_id == 'memory':
+            if hw_id == 'config':
+                continue
+            elif hw_id == 'memory':
                 cl_name = "RasdMemory"
             elif hw_id == 'cpus':
                 cl_name = "RasdCpus"
@@ -1049,6 +1126,7 @@ class OVF(object):
                 rasd_items[hw_id] = rasd_item
             except AttributeError:
                 print(f"no class {cl_name}")
+                raise
         return rasd_items
 
 
@@ -1072,7 +1150,7 @@ class OVF(object):
 
     @staticmethod
     def _disk_info(filename):
-        out = subprocess.check_output([VMDK_CONVERT, "-i", filename]).decode("UTF-8")
+        out = subprocess.check_output([VMDK_CONVERT, "-i", "--detailed", filename]).decode("UTF-8")
         return json.loads(out)
 
 
@@ -1104,7 +1182,7 @@ class OVF(object):
         network_section = ET.Element('{%s}NetworkSection' % NS_OVF)
         network_section.append(xml_text_element('{%s}Info' % NS_OVF, "Virtual Networks"))
         for nw_id, nw in self.networks.items():
-            network_section.append(nw.xml_item())        
+            network_section.append(nw.xml_item())
         envelope.append(network_section)
 
         # VirtualSystem
@@ -1143,8 +1221,7 @@ class OVF(object):
         for xcfg in self.extra_configs:
             hw.append(xcfg.xml_item())
 
-        for key, val in sorted(self.hardware_config.items()):
-            hw.append(xml_config(key, val))
+        hw.extend(xml_configs(self.hardware_config))
 
         if self.products:
             for p in self.products:
@@ -1216,6 +1293,42 @@ class OVF(object):
                 f.write(f"{hash_type.upper()}({fname})= {hash}\n")
 
 
+    def sign_manifest(self, keyfile, ovf_file=None, mf_file=None, sign_alg="sha512"):
+        if ovf_file == None:
+            ovf_file = f"{self.name}.ovf"
+        if mf_file == None:
+            mf_file = f"{self.name}.mf"
+        cert_file = os.path.splitext(ovf_file)[0] + ".cert"
+
+        with open(cert_file, "wt") as f:
+            signature = subprocess.check_output(["openssl", "dgst", f"-{sign_alg}", "-sign", keyfile, "-out", "-", mf_file])
+            f.write(f"{sign_alg.upper()}({mf_file})= {signature.hex()}\n")
+
+            with open(keyfile, "rt") as fin:
+                do_copy = False
+                for line in fin:
+                    if (line.startswith("-----BEGIN CERTIFICATE")):
+                        do_copy = True
+                    if do_copy:
+                        f.write(line)
+                    if (line.startswith("-----END CERTIFICATE")):
+                        break
+                assert do_copy, f"no certificate found in {keyfile}"
+
+
+    def sign_manifest_external(self, script_format, keyfile, ovf_file=None, mf_file=None, sign_alg="sha512"):
+        if ovf_file == None:
+            ovf_file = f"{self.name}.ovf"
+        if mf_file == None:
+            mf_file = f"{self.name}.mf"
+        cert_file = os.path.splitext(ovf_file)[0] + ".cert"
+
+        script = script_format.format(ovf_file=ovf_file, mf_file=mf_file, sign_alg=sign_alg, cert_file=cert_file, keyfile=keyfile)
+        subprocess.check_call(script, shell=True)
+        assert os.path.isfile(cert_file), f"certificate file {cert_file} not created"
+        assert os.path.getsize(cert_file) > 0, f"certificate file {cert_file} is empty"
+
+
 def usage():
     print(f"Usage: {sys.argv[0]} -i|--input-file <input file> -o|--output-file <output file> [--format ova|ovf|dir] [-q] [-h]")
     print("")
@@ -1225,6 +1338,11 @@ def usage():
     print("  -f, --format ova|ovf|dir    output format")
     print("  -m, --manifest              create manifest file along with ovf (default true for output formats ova and dir)")
     print("  --checksum-type sha1|sha256|sha512  set the checksum type for the manifest. Must be sha1, sha256 or sha512.")
+    print("  --sign <keyfile>            sign the manifest file with the given keyfile")
+    print("  --sign-alg sha1|sha256|sha512  set the signature algorithm for the manifest. Must be sha1, sha256 or sha512. Default is the same as the checksum-type.")
+    print("  --sign-script <script>      sign the manifest file with the given script")
+    print("  --tar-format gnu|posix      set the tar format for the ova file. Must be gnu or posix. Default is gnu.")
+    print("  --vmdk-convert <path>       set the path to the vmdk-convert tool (optional)")
     print("  -q                          quiet mode")
     print("  -h                          print help")
     print("")
@@ -1235,6 +1353,22 @@ def usage():
     print("")
     print("Specifying the format is optional if the output file name ends with '.ovf' or '.ova'")
     print("")
+    print("Sign options:")
+    print("    To sign the manifest file, you can use the --sign option to provide a keyfile, or the --sign-script option to provide a script.")
+    print("    The keyfile should be a PEM encoded certificate and key file.")
+    print("    Optionally, the signature algorithm (sha1, sha256, sha512) can be set with the --sign-alg option. Default is the same as the checksum-type.")
+    print("    If no sign-script is given, ova-compose will use the keyfile to sign the manifest using openssl.")
+    print("    If a sign-script is given, it should be a shell python style format string that takes the following arguments:")
+    print("    - {ovf_file}: the path to the ovf file")
+    print("    - {mf_file}: the path to the manifest file")
+    print("    - {sign_alg}: the signature algorithm")
+    print("    - {cert_file}: the path to the certificate file (the signature to be included)")
+    print("    - {keyfile}: the path to the keyfile")
+    print("    These arguments will be set by the ova-compose tool and passed to the script.")
+    print("    It is expected that the script signs the file given by {mf_file} and writes the signature to {cert_file}.")
+    print("    Refer to the DMTF DSP2434 specification for more details on the signature format.")
+    print("    The script should be executable and should be in the PATH or the full path to the script should be provided.")
+    print("")
     print("Example usage:")
     print(f"  {sys.argv[0]} -i photon.yaml -o photon.ova")
 
@@ -1244,7 +1378,7 @@ def yaml_param(loader, node):
     default = None
     key = node.value
 
-    assert type(key) is str, f"param name must be a string"
+    assert type(key) is str, f"param name {key} must be a string"
 
     if '=' in key:
         key, default = [t.strip() for t in key.split('=', maxsplit=1)]
@@ -1265,10 +1399,15 @@ def main():
     do_manifest = False
     params = {}
     checksum_type = "sha256"
+    sign_keyfile = None
+    sign_alg = None
+    sign_script = None
     tar_format = "gnu"
 
     try:
-        opts, args = getopt.getopt(sys.argv[1:], 'f:hi:mo:q', longopts=['format=', 'input-file=', 'manifest', 'output-file=', 'param=', 'checksum-type=', 'tar-format=', 'vmdk-convert='])
+        opts, args = getopt.getopt(sys.argv[1:],
+            'f:hi:mo:q',
+            longopts=['format=', 'input-file=', 'manifest', 'output-file=', 'param=', 'checksum-type=', 'sign=', 'sign-alg=', 'sign-script=', 'tar-format=', 'vmdk-convert='])
     except:
         print ("invalid option")
         sys.exit(2)
@@ -1294,6 +1433,12 @@ def main():
             VMDK_CONVERT = a
         elif o in ['-q']:
             do_quiet = True
+        elif o in ['-s', '--sign']:
+            sign_keyfile = a
+        elif o in ['--sign-alg']:
+            sign_alg = a
+        elif o in ['--sign-script']:
+            sign_script = a
         elif o in ['-h']:
             usage()
             sys.exit(0)
@@ -1304,6 +1449,12 @@ def main():
     assert output_file != None, "no output file/directory specified"
 
     assert checksum_type in ["sha1", "sha512", "sha256"], f"checksum-type '{checksum_type}' is invalid"
+    if sign_alg is None:
+        sign_alg = checksum_type
+    assert sign_alg in ["sha1", "sha512", "sha256"], f"checksum-type '{sign_alg}' is invalid"
+
+    if sign_keyfile is not None:
+        sign_keyfile = os.path.abspath(sign_keyfile)
 
     if config_file != None:
         f = open(config_file, 'r')
@@ -1343,6 +1494,11 @@ def main():
         ovf.write_xml(ovf_file=ovf_file)
         if do_manifest:
             ovf.write_manifest(ovf_file=ovf_file, mf_file=mf_file, hash_type=checksum_type)
+            if sign_keyfile is not None or sign_script is not None:
+                if sign_script is None:
+                    ovf.sign_manifest(sign_keyfile, ovf_file=ovf_file, mf_file=mf_file, sign_alg=sign_alg)
+                else:
+                    ovf.sign_manifest_external(sign_script, sign_keyfile, ovf_file=ovf_file, mf_file=mf_file, sign_alg=sign_alg)
     elif output_format == "ova" or output_format == "dir":
         pwd = os.getcwd()
         tmpdir = tempfile.mkdtemp(prefix=f"{basename}-", dir=pwd)
@@ -1358,6 +1514,13 @@ def main():
                 all_files.append(dst)
 
             ovf.write_manifest(ovf_file=ovf_file, mf_file=mf_file, hash_type=checksum_type)
+            if sign_keyfile is not None or sign_script is not None:
+                if sign_script is None:
+                    ovf.sign_manifest(sign_keyfile, ovf_file=ovf_file, mf_file=mf_file, sign_alg=sign_alg)
+                else:
+                    ovf.sign_manifest_external(sign_script, sign_keyfile, ovf_file=ovf_file, mf_file=mf_file, sign_alg=sign_alg)
+                cert_file = os.path.splitext(ovf_file)[0] + ".cert"
+                all_files.append(cert_file)
 
             if output_format == "ova":
                 ret = subprocess.check_call(["tar", f"--format={tar_format}", "-h",
