@@ -31,6 +31,7 @@ typedef struct {
     int (*abort)(DiskInfo *self);
     ssize_t (*copyDisk)(DiskInfo *self, DiskInfo *src, int numThreads);
     bool (*checkGrainOrder)(DiskInfo *self);  /* Returns true if grains are ordered in the grain table */
+    char *(*getDescriptor)(DiskInfo *self);   /* Returns the descriptor file content if available, NULL otherwise */
 } DiskInfoVMT;
 
 struct DiskInfo {
@@ -42,6 +43,6 @@ extern char *toolsVersion; /* toolsVersion in metadata */
 DiskInfo *Flat_Open(const char *fileName);
 DiskInfo *Flat_Create(const char *fileName, off_t capacity);
 DiskInfo *Sparse_Open(const char *fileName);
-DiskInfo *StreamOptimized_Create(const char *fileName, off_t capacity, int compressionLevel, bool doReorder);
+DiskInfo *StreamOptimized_Create(const char *fileName, off_t capacity, int compressionLevel, bool doReorder, int sectorSize);
 
 #endif /* _DISKINFO_H_ */
