@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/stat.h>
 
 #include <zlib.h>
 
@@ -67,6 +68,7 @@ typedef struct SparseVmdkWriter {
     char *fileName;
     int compressionLevel;
     bool doReorder;
+    uint32_t sectorSize; /* we can only know for sure when writing, therefore it's here */
 } SparseVmdkWriter;
 
 typedef struct StreamOptimizedDiskInfo {
@@ -77,8 +79,10 @@ typedef struct StreamOptimizedDiskInfo {
 
 typedef struct SparseDiskInfo {
     DiskInfo hdr;
+    bool hasFooter;
     SparseExtentHeader diskHdr;
     SparseGTInfo gtInfo;
+    char *descriptor;  // Added to store the descriptor file content
     int fd;
 } SparseDiskInfo;
 
@@ -219,6 +223,7 @@ setSparseExtentHeader(SparseExtentHeaderOnDisk *dst,
 static char *
 makeDiskDescriptorFile(const char *fileName,
                        uint64_t capacity,
+                       uint32_t sectorSize,
                        uint32_t cid)
 {
     static const char ddfTemplate[] =
@@ -240,21 +245,33 @@ makeDiskDescriptorFile(const char *fileName,
 "ddb.geometry.cylinders = \"%u\"\n"
 "ddb.geometry.heads = \"255\"\n" /* 255/63 is good for anything bigger than 4GB. */
 "ddb.geometry.sectors = \"63\"\n"
+"%s"
 "ddb.adapterType = \"lsilogic\"\n"
 "ddb.toolsInstallType = \"4\"\n" /* unmanaged (open-vm-tools) */
 "ddb.toolsVersion = \"%s\""; /* open-vm-tools version */
 
     unsigned int cylinders;
-    char *ret;
+    char *secEntries = NULL;
+    char *ret = NULL;
 
     if (capacity > 65535 * 255 * 63) {
         cylinders = 65535;
     } else {
         cylinders = CEILING(capacity, 255 * 63);
     }
-    if (asprintf(&ret, ddfTemplate, cid, (long long int)capacity, fileName, (uint32_t)mrand48(), (uint32_t)mrand48(), (uint32_t)mrand48(), cid, cylinders, toolsVersion) == -1) {
+
+    if (sectorSize > 0) {
+        if(asprintf(&secEntries, "ddb.logicalSectorSize = \"%d\"\nddb.physicalSectorSize = \"%d\"\n", sectorSize, sectorSize) == -1)
+            return NULL;
+    }
+
+    if (asprintf(&ret, ddfTemplate, cid, (long long int)capacity, fileName, (uint32_t)mrand48(), (uint32_t)mrand48(), (uint32_t)mrand48(), cid, cylinders, secEntries ? secEntries : "", toolsVersion) == -1) {
         return NULL;
     }
+
+    if (secEntries)
+        free(secEntries);
+
     return ret;
 }
 
@@ -754,7 +771,7 @@ writeGrainTables(int fd, const SparseExtentHeader *hdr, const SparseGTInfo *gtIn
 }
 
 static bool
-writeDescriptor(int fd, const SparseExtentHeader *hdr)
+writeDescriptor(int fd, const SparseExtentHeader *hdr, uint32_t sectorSize)
 {
     uint32_t cid;
     char *descFile;
@@ -763,8 +780,8 @@ writeDescriptor(int fd, const SparseExtentHeader *hdr)
     do {
         cid = mrand48();
     } while (cid == 0xFFFFFFFFU || cid == 0xFFFFFFFEU);
-    
-    descFile = makeDiskDescriptorFile("disk", hdr->capacity, cid);
+
+    descFile = makeDiskDescriptorFile("disk", hdr->capacity, sectorSize, cid);
     if (!descFile) {
         fprintf(stderr, "Failed to create descriptor file\n");
         return false;
@@ -830,7 +847,7 @@ rewriteGrains(StreamOptimizedDiskInfo *sodi,
     /* Process grains in order */
     for (grainNr = 0; grainNr < sodi->writer.gtInfo.GTEs; grainNr++) {
         uint32_t oldSector = __le32_to_cpu(srcGT->gt[grainNr]);
-        
+
         /* Skip empty (0) and zero (1) grains */
         if (oldSector <= 1) {
             dstGT->gt[grainNr] = __cpu_to_le32(oldSector);
@@ -838,7 +855,7 @@ rewriteGrains(StreamOptimizedDiskInfo *sodi,
         }
 
 #ifdef DEBUG
-        printf("DEBUG: Processing grain %llu: old sector=%u, new sector=%u (sequential)\n", 
+        printf("DEBUG: Processing grain %llu: old sector=%u, new sector=%u (sequential)\n",
                (unsigned long long)grainNr, oldSector, nextSector);
 #endif
 
@@ -850,7 +867,7 @@ rewriteGrains(StreamOptimizedDiskInfo *sodi,
 
         uint32_t cmpSize;
         uint32_t hdrlen;
-        
+
         /* Parse header based on format */
         if (sodi->diskHdr.flags & SPARSEFLAG_EMBEDDED_LBA) {
             SparseGrainLBAHeaderOnDisk *hdr = (SparseGrainLBAHeaderOnDisk *)readBuf;
@@ -862,7 +879,7 @@ rewriteGrains(StreamOptimizedDiskInfo *sodi,
         }
 
 #ifdef DEBUG
-        printf("DEBUG: Grain %llu has compressed size %u, header length %u\n", 
+        printf("DEBUG: Grain %llu has compressed size %u, header length %u\n",
                (unsigned long long)grainNr, cmpSize, hdrlen);
 #endif
 
@@ -886,7 +903,7 @@ rewriteGrains(StreamOptimizedDiskInfo *sodi,
         size_t totalSize = VMDK_SECTOR_SIZE + remainingLength;
 
 #ifdef DEBUG
-        printf("DEBUG: Writing grain %llu to temp file at sector %u (sequential), size %zu\n", 
+        printf("DEBUG: Writing grain %llu to temp file at sector %u (sequential), size %zu\n",
                (unsigned long long)grainNr, nextSector, totalSize);
 #endif
 
@@ -1184,7 +1201,7 @@ StreamOptimizedClose(DiskInfo *self)
             goto failAll;
         }
     }
-    
+
     if (!writeEOS(&sodi->writer)) {
         fprintf(stderr, "Failed to write EOS marker\n");
         goto failAll;
@@ -1193,7 +1210,7 @@ StreamOptimizedClose(DiskInfo *self)
         fprintf(stderr, "Failed to write grain tables\n");
         goto failAll;
     }
-    if (!writeDescriptor(sodi->writer.fd, &sodi->diskHdr)) {
+    if (!writeDescriptor(sodi->writer.fd, &sodi->diskHdr, sodi->writer.sectorSize)) {
         fprintf(stderr, "Failed to write descriptor\n");
         goto failAll;
     }
@@ -1223,7 +1240,7 @@ static DiskInfoVMT streamOptimizedVMT = {
 };
 
 DiskInfo *
-StreamOptimized_Create(const char *fileName, off_t capacity, int compressionLevel, bool doReorder)
+StreamOptimized_Create(const char *fileName, off_t capacity, int compressionLevel, bool doReorder, int sectorSize)
 {
     StreamOptimizedDiskInfo *sodi;
 
@@ -1253,6 +1270,7 @@ StreamOptimized_Create(const char *fileName, off_t capacity, int compressionLeve
     }
     sodi->writer.compressionLevel = compressionLevel;
     sodi->writer.doReorder = doReorder;
+    sodi->writer.sectorSize = sectorSize;
 
     sodi->diskHdr.descriptorOffset = sodi->diskHdr.overHead;
     sodi->diskHdr.descriptorSize = 20;
@@ -1482,6 +1500,12 @@ SparseClose(DiskInfo *self)
 
     free(sdi->gtInfo.gd);
     fd = sdi->fd;
+
+    // Free the descriptor if it exists
+    if (sdi->descriptor) {
+        free(sdi->descriptor);
+    }
+
     free(sdi);
     return close(fd);
 }
@@ -1501,7 +1525,7 @@ areGrainsOrdered(const SparseGTInfo *gtInfo)
     /* Iterate through the grain table */
     for (i = 0; i < gtInfo->GTEs; i++) {
         uint32_t sector = __le32_to_cpu(gtInfo->gt[i]);
-        
+
         /* Skip empty grains (marked as 0) and zero grains (marked as 1) */
         if (sector <= 1) {
             continue;
@@ -1549,6 +1573,17 @@ SparseCheckGrainOrder(DiskInfo *self)
     return areSparseGrainsOrdered(sdi);
 }
 
+static char *
+SparseGetDescriptor(DiskInfo *self)
+{
+    SparseDiskInfo *sdi = getSDI(self);
+    if (!sdi) {
+        return NULL;
+    }
+
+    return sdi->descriptor;
+}
+
 static DiskInfoVMT sparseVMT = {
     .getCapacity = SparseGetCapacity,
     .pread = SparsePread,
@@ -1557,9 +1592,89 @@ static DiskInfoVMT sparseVMT = {
     .close = SparseClose,
     .abort = SparseClose,
     .copyDisk = NULL,
-    .checkGrainOrder = SparseCheckGrainOrder
+    .checkGrainOrder = SparseCheckGrainOrder,
+    .getDescriptor = SparseGetDescriptor
 };
 
+static char *
+getDescriptorFile(int fd, const SparseExtentHeader *hdr)
+{
+    char *descriptor = NULL;
+    size_t descriptorSize;
+
+    if (hdr->descriptorOffset == 0 || hdr->descriptorSize == 0) {
+        fprintf(stderr, "No descriptor found in VMDK file\n");
+        return NULL;
+    }
+
+    descriptorSize = hdr->descriptorSize * VMDK_SECTOR_SIZE;
+    descriptor = malloc(descriptorSize + 1); // +1 for null terminator
+    if (!descriptor) {
+        fprintf(stderr, "Failed to allocate memory for descriptor\n");
+        return NULL;
+    }
+
+    if (!safePread(fd, descriptor, descriptorSize, hdr->descriptorOffset * VMDK_SECTOR_SIZE)) {
+        fprintf(stderr, "Failed to read descriptor from VMDK file\n");
+        free(descriptor);
+        return NULL;
+    }
+
+    // Ensure null termination
+    descriptor[descriptorSize] = '\0';
+
+    return descriptor;
+}
+
+/* Sparse VMDK files optionally have a footer near the end of the file,
+   containing the SparseExtentHeader. If present, it overrides the header
+   at the beginning of the file.
+ 
+   A valid footer is preceded a meta data marker with type GRAIN_MARKER_FOOTER.
+
+   If a valid footer is found, it is copied to the memory pointed to by footer.
+*/
+
+static bool
+readSparseFooter(int fd, SparseExtentHeaderOnDisk *footer)
+{
+    off_t offset;
+    struct stat sb;
+    SparseMetaDataMarkerOnDisk metaMarker;
+    SparseExtentHeaderOnDisk onDisk;
+
+    if (fstat(fd, &sb) != 0) {
+        fprintf(stderr, "fstat failed: %s (fd=%d)\n", strerror(errno), fd);
+        return false;
+    }
+
+    offset = (sb.st_size - 3 * VMDK_SECTOR_SIZE) & ~(VMDK_SECTOR_SIZE - 1);
+    if (!safePread(fd, &metaMarker, sizeof metaMarker, offset)) {
+        return false;
+    }
+
+    if (metaMarker.size != 0) {
+        return false;
+    }
+
+    if (__le32_to_cpu(metaMarker.type) != GRAIN_MARKER_FOOTER) {
+        return false;
+    }
+
+    offset = (sb.st_size - 2 * VMDK_SECTOR_SIZE) & ~(VMDK_SECTOR_SIZE - 1);
+    if (!safePread(fd, &onDisk, sizeof onDisk, offset)) {
+        return false;
+    }
+
+    if (!checkSparseExtentHeader(&onDisk)) {
+        return false;
+    }
+
+    memcpy(footer, &onDisk, sizeof onDisk);
+
+    return true;
+}
+
 DiskInfo *
 Sparse_Open(const char *fileName)
 {
@@ -1580,11 +1695,15 @@ Sparse_Open(const char *fileName)
     if (!checkSparseExtentHeader(&onDisk)) {
         goto failFd;
     }
+
     sdi = malloc(sizeof *sdi);
     if (!sdi) {
         goto failFd;
     }
     memset(sdi, 0, sizeof *sdi);
+
+    sdi->hasFooter = readSparseFooter(fd, &onDisk);
+
     sdi->fd = fd;
     if (!getSparseExtentHeader(&sdi->diskHdr, &onDisk)) {
         goto failSdi;
@@ -1611,6 +1730,10 @@ Sparse_Open(const char *fileName)
     if (CoalescedPreaderExec(&cp)) {
         goto failDF;
     }
+
+    // Read the descriptor file and store it in the SparseDiskInfo
+    sdi->descriptor = getDescriptorFile(fd, &sdi->diskHdr);
+
     return &sdi->hdr;
 
 failDF:
